@@ -1,4 +1,4 @@
-/* $Id: actions.cc,v 1.17 2005/12/27 22:56:35 helly Exp $ */
+/* $Id: actions.cc 674 2007-04-16 21:39:11Z helly $ */
 #include <time.h>
 #include <string.h>
 #include <iostream>
@@ -14,6 +14,11 @@ namespace re2c
 
 void Symbol::ClearTable()
 {
+	for (SymbolTable::iterator it = symbol_table.begin(); it != symbol_table.end(); ++it)
+	{
+		delete it->second;
+	}
+	
 	symbol_table.clear();
 }
 
@@ -59,7 +64,7 @@ void showIns(std::ostream &o, const Ins &i, const Ins &base)
 		break;
 
 		case CTXT:
-		o << "term " << ((RuleOp*) i.i.link)->accept;
+		o << "ctxt";
 		break;
 
 		case TERM:
@@ -75,7 +80,7 @@ uint RegExp::fixedLength()
 	return ~0;
 }
 
-char *NullOp::type = "NullOp";
+const char *NullOp::type = "NullOp";
 
 void NullOp::calcSize(Char*)
 {
@@ -233,7 +238,7 @@ MatchOp *merge(MatchOp *m1, MatchOp *m2)
 	return new MatchOp(doUnion(m1->match, m2->match));
 }
 
-char *MatchOp::type = "MatchOp";
+const char *MatchOp::type = "MatchOp";
 
 void MatchOp::display(std::ostream &o) const
 {
@@ -371,7 +376,7 @@ RegExp *mkAlt(RegExp *e1, RegExp *e2)
 	return doAlt(merge(m1, m2), doAlt(e1, e2));
 }
 
-char *AltOp::type = "AltOp";
+const char *AltOp::type = "AltOp";
 
 void AltOp::calcSize(Char *rep)
 {
@@ -408,7 +413,7 @@ void AltOp::split(CharSet &s)
 	exp2->split(s);
 }
 
-char *CatOp::type = "CatOp";
+const char *CatOp::type = "CatOp";
 
 void CatOp::calcSize(Char *rep)
 {
@@ -440,7 +445,7 @@ void CatOp::split(CharSet &s)
 	exp2->split(s);
 }
 
-char *CloseOp::type = "CloseOp";
+const char *CloseOp::type = "CloseOp";
 
 void CloseOp::calcSize(Char *rep)
 {
@@ -461,7 +466,7 @@ void CloseOp::split(CharSet &s)
 	exp->split(s);
 }
 
-char *CloseVOp::type = "CloseVOp";
+const char *CloseVOp::type = "CloseVOp";
 
 void CloseVOp::calcSize(Char *rep)
 {
@@ -513,96 +518,218 @@ void CloseVOp::split(CharSet &s)
 
 RegExp *expr(Scanner &);
 
-uchar Scanner::unescape(SubStr &s) const
+uint Scanner::unescape(SubStr &s) const
 {
+	static const char * hex = "0123456789abcdef";
+	static const char * oct = "01234567";
+
 	s.len--;
-	uchar c;
+	uint c, ucb = 0;
 
 	if ((c = *s.str++) != '\\' || s.len == 0)
-		return xlat[c];
+	{
+		return xlat(c);
+	}
 
 	s.len--;
 
 	switch (c = *s.str++)
 	{
+		case 'n': return xlat('\n');
+		case 't': return xlat('\t');
+		case 'v': return xlat('\v');
+		case 'b': return xlat('\b');
+		case 'r': return xlat('\r');
+		case 'f': return xlat('\f');
+		case 'a': return xlat('\a');
 
-		case 'n':
-		return xlat['\n'];
-
-		case 't':
-		return xlat['\t'];
-
-		case 'v':
-		return xlat['\v'];
-
-		case 'b':
-		return xlat['\b'];
-
-		case 'r':
-		return xlat['\r'];
-
-		case 'f':
-		return xlat['\f'];
-
-		case 'a':
-		return xlat['\a'];
-		
 		case 'x':
 		{
-			static const char * hex = "0123456789abcdef";
-			char *p1, *p2;
-
-			if (s.len < 2 || !(p1 = strchr(hex, tolower(s.str[0]))) 
-			              || !(p2 = strchr(hex, tolower(s.str[1]))))
+			if (s.len < 2)
 			{
-				fatal("Illegal hexadecimal character code");
+				fatal(s.ofs()+s.len, "Illegal hexadecimal character code, two hexadecimal digits are required");
+				return ~0;
 			}
-			s.len -= 2;
-			s.str += 2;
 			
-			uchar v = (uchar)((p1 - hex) << 4) + (uchar)(p2 - hex);
+			const char *p1 = strchr(hex, tolower(s.str[0]));
+			const char *p2 = strchr(hex, tolower(s.str[1]));
 
-			return v;
+			if (!p1 || !p2)
+			{
+				fatal(s.ofs()+(p1?1:0), "Illegal hexadecimal character code");
+				return ~0;
+			}
+			else
+			{
+				s.len -= 2;
+				s.str += 2;
+				
+				uint v = (uint)((p1 - hex) << 4) 
+				       + (uint)((p2 - hex));
+	
+				return v;
+			}
+		}
+
+		case 'U':
+		{
+			if (s.len < 8)
+			{
+				fatal(s.ofs()+s.len, "Illegal unicode character, eight hexadecimal digits are required");
+				return ~0;
+			}
+
+			uint l = 0;
+						
+			if (s.str[0] == '0')
+			{
+				l++;
+				if (s.str[1] == '0')
+				{
+					l++;
+					if (s.str[2] == '0' || (s.str[2] == '1' && uFlag))
+					{
+						l++;
+						if (uFlag) {
+							const char *u3 = strchr(hex, tolower(s.str[2]));
+							const char *u4 = strchr(hex, tolower(s.str[3]));
+							if (u3 && u4)
+							{
+								ucb = (uint)((u3 - hex) << 20)
+							        + (uint)((u4 - hex) << 16);
+								l++;
+							}
+						}
+						else if (s.str[3] == '0')
+						{
+							l++;
+						}
+					}
+				}
+			}
+
+			if (l != 4)
+			{
+				fatal(s.ofs()+l, "Illegal unicode character, eight hexadecimal digits are required");
+			}
+
+			s.len -= 4;
+			s.str += 4;
+			
+			// no break;
+		}
+		case 'X':
+		case 'u':
+		{
+			if (s.len < 4)
+			{
+				fatal(s.ofs()+s.len, 
+					c == 'X'
+					? "Illegal hexadecimal character code, four hexadecimal digits are required"
+					: "Illegal unicode character, four hexadecimal digits are required");
+				return ~0;
+			}
+			
+			const char *p1 = strchr(hex, tolower(s.str[0]));
+			const char *p2 = strchr(hex, tolower(s.str[1]));
+			const char *p3 = strchr(hex, tolower(s.str[2]));
+			const char *p4 = strchr(hex, tolower(s.str[3]));
+
+			if (!p1 || !p2 || !p3 || !p4)
+			{
+				fatal(s.ofs()+(p1?1:0)+(p2?1:0)+(p3?1:0), 
+					c == 'X'
+					? "Illegal hexadecimal character code, non hexxdecimal digit found"
+					: "Illegal unicode character, non hexadecimal digit found");
+				return ~0;
+			}
+			else
+			{
+				s.len -= 4;
+				s.str += 4;
+				
+				uint v = (uint)((p1 - hex) << 12) 
+				       + (uint)((p2 - hex) <<  8)
+				       + (uint)((p3 - hex) <<  4)
+				       + (uint)((p4 - hex))
+				       + ucb;
+	
+				if (v >= nRealChars)
+				{
+					fatal(s.ofs(),
+						c == 'X'
+						? "Illegal hexadecimal character code, out of range"
+						: "Illegal unicode character, out of range");
+				}
+	
+				return v;
+			}
+		}
+
+		case '4':
+		case '5':
+		case '6':
+		case '7':
+		{
+			fatal(s.ofs()-1, "Illegal octal character code, first digit must be 0 thru 3");
+			return ~0;
 		}
 
 		case '0':
-
 		case '1':
-
 		case '2':
-
 		case '3':
-
-		case '4':
-
-		case '5':
-
-		case '6':
-
-		case '7':
 		{
-			static const char * oct = "01234567";
-			char *p0, *p1, *p2;
-
-			if (s.len < 2 || !(p0 = strchr(oct, c)) || c > '3'
-			              || !(p1 = strchr(oct, s.str[0])) 
-			              || !(p2 = strchr(oct, s.str[1])))
+			if (s.len < 2)
 			{
-				fatal("Illegal octal character code");
+				fatal(s.ofs()+s.len, "Illegal octal character code, three octal digits are required");
+				return ~0;
 			}
-			s.len -= 2;
-			s.str += 2;
-			
-			uchar v = (uchar)((p0 - oct) << 6) + (uchar)((p1 - oct) << 3) + (uchar)(p2 - oct);
 
-			return v;
+			const char *p0 = strchr(oct, c);
+			const char *p1 = strchr(oct, s.str[0]);
+			const char *p2 = strchr(oct, s.str[1]);
+
+			if (!p0 || !p1 || !p2)
+			{
+				fatal(s.ofs()+(p1?1:0), "Illegal octal character code, non octal digit found");
+				return ~0;
+			}
+			else
+			{
+				s.len -= 2;
+				s.str += 2;
+				
+				uint v = (uint)((p0 - oct) << 6) + (uint)((p1 - oct) << 3) + (uint)(p2 - oct);
+	
+				return v;
+			}
 		}
 
 		default:
-		return xlat[c];
+		return xlat(c);
 	}
 }
 
+std::string& Scanner::unescape(SubStr& str_in, std::string& str_out) const
+{
+	str_out.clear();
+
+	while(str_in.len)
+	{
+		uint c = unescape(str_in);
+		
+		if (c > 0xFF)
+		{
+			fatal(str_in.ofs(), "Illegal character");
+		}
+
+		str_out += static_cast<char>(c);
+	}
+
+	return str_out;
+}
+
 Range * Scanner::getRange(SubStr &s) const
 {
 	uint lb = unescape(s), ub, xlb, xub, c;
@@ -624,18 +751,18 @@ Range * Scanner::getRange(SubStr &s) const
 			ub = tmp;
 		}
 		
-		xlb = xlat[lb];
-		xub = xlat[ub];
+		xlb = xlat(lb);
+		xub = xlat(ub);
 		
 		for(c = lb; c <= ub; c++)
 		{
-			if (!(xlb <= xlat[c] && xlat[c] <= ub))
+			if (!(xlb <= xlat(c) && xlat(c) <= ub))
 			{
 				/* range doesn't work */
 				Range * r = new Range(xlb, xlb + 1);
 				for (c = lb + 1; c <= ub; c++)
 				{
-					r = doUnion(r, new Range(xlat[c], xlat[c] + 1));
+					r = doUnion(r, new Range(xlat(c), xlat(c) + 1));
 				}
 				return r;
 			}
@@ -677,14 +804,14 @@ RegExp * Scanner::strToCaseInsensitiveRE(SubStr s) const
 	if (s.len == 0)
 		return new NullOp;
 
-	uchar c = unescape(s);
+	uint c = unescape(s);
 
 	RegExp *re, *reL, *reU;
 
 	if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'))
 	{
-		reL = matchChar(xlat[tolower(c)]);
-		reU = matchChar(xlat[toupper(c)]);
+		reL = matchChar(xlat(tolower(c)));
+		reU = matchChar(xlat(toupper(c)));
 		re = mkAlt(reL, reU);
 	}
 	else
@@ -694,12 +821,12 @@ RegExp * Scanner::strToCaseInsensitiveRE(SubStr s) const
 
 	while (s.len > 0)
 	{
-		uchar c = unescape(s);
+		uint c = unescape(s);
 
 		if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z'))
 		{
-			reL = matchChar(xlat[tolower(c)]);
-			reU = matchChar(xlat[toupper(c)]);
+			reL = matchChar(xlat(tolower(c)));
+			reU = matchChar(xlat(toupper(c)));
 			re = new CatOp(re, mkAlt(reL, reU));
 		}
 		else
@@ -732,7 +859,7 @@ RegExp * Scanner::invToRE(SubStr s) const
 	s.len--;
 	s.str++;
 	
-	RegExp * any = ranToRE(SubStr("[\\000-\\377]"));
+	RegExp * any = ranToRE(SubStr(wFlag ? "[\\X0000-\\XFFFF]" : "[\\000-\\377]"));
 
 	if (s.len <= 2)
 	{
@@ -750,8 +877,8 @@ RegExp * Scanner::invToRE(SubStr s) const
 
 RegExp * Scanner::mkDot() const
 {
-	RegExp * any = ranToRE(SubStr("[\\000-\\377]"));
-	RegExp * ran = matchChar(xlat['\n']);
+	RegExp * any = ranToRE(SubStr(wFlag ? "[\\X0000-\\XFFFF]" : "[\\000-\\377]"));
+	RegExp * ran = matchChar(xlat('\n'));
 	RegExp * inv = mkDiff(any, ran);
 	
 	delete ran;
@@ -760,10 +887,15 @@ RegExp * Scanner::mkDot() const
 	return inv;
 }
 
-char *RuleOp::type = "RuleOp";
+const char *RuleOp::type = "RuleOp";
 
 RuleOp::RuleOp(RegExp *e, RegExp *c, Token *t, uint a)
-		: exp(e), ctx(c), ins(NULL), accept(a), code(t)
+	: exp(e)
+	, ctx(c)
+	, ins(NULL)
+	, accept(a)
+	, code(t)
+	, line(0)
 {
 	;
 }
@@ -772,7 +904,7 @@ void RuleOp::calcSize(Char *rep)
 {
 	exp->calcSize(rep);
 	ctx->calcSize(rep);
-	size = exp->size + ctx->size + 1;
+	size = exp->size + (ctx->size ? ctx->size + 2 : 1);
 }
 
 void RuleOp::compile(Char *rep, Ins *i)
@@ -780,8 +912,14 @@ void RuleOp::compile(Char *rep, Ins *i)
 	ins = i;
 	exp->compile(rep, &i[0]);
 	i += exp->size;
-	ctx->compile(rep, &i[0]);
-	i += ctx->size;
+	if (ctx->size)
+	{
+		i->i.tag = CTXT;
+		i->i.link = &i[1];
+		i++;
+		ctx->compile(rep, &i[0]);
+		i += ctx->size;
+	}
 	i->i.tag = TERM;
 	i->i.link = this;
 }
@@ -792,8 +930,6 @@ void RuleOp::split(CharSet &s)
 	ctx->split(s);
 }
 
-extern void printSpan(std::ostream&, uint, uint);
-
 void optimize(Ins *i)
 {
 	while (!isMarked(i))
@@ -839,31 +975,51 @@ void optimize(Ins *i)
 
 void genCode(std::ostream& o, RegExp *re)
 {
-	CharSet cs;
-	uint j;
-	memset(&cs, 0, sizeof(cs));
+	genCode(o, 0, re);
+}
 
-	for (j = 0; j < nChars; ++j)
+CharSet::CharSet()
+	: fix(0)
+	, freeHead(0)
+	, freeTail(0)
+	, rep(new CharPtr[nRealChars])
+	, ptn(new CharPtn[nRealChars])
+{
+	for (uint j = 0; j < nRealChars; ++j)
 	{
-		cs.rep[j] = &cs.ptn[0];
-		cs.ptn[j].nxt = &cs.ptn[j + 1];
+		rep[j] = &ptn[0];
+		ptn[j].nxt = &ptn[j + 1]; /* wrong for j=nRealChars but will be corrected below */
+		ptn[j].card = 0;
 	}
 
-	cs.freeHead = &cs.ptn[1];
-	*(cs.freeTail = &cs.ptn[nChars - 1].nxt) = NULL;
-	cs.ptn[0].card = nChars;
-	cs.ptn[0].nxt = NULL;
+	freeHead = &ptn[1];
+	*(freeTail = &ptn[nRealChars - 1].nxt) = NULL;
+	ptn[0].card = nRealChars;
+	ptn[0].nxt = NULL;
+}
+	
+CharSet::~CharSet()
+{
+	delete[] rep;
+	delete[] ptn;
+}
+
+void genCode(std::ostream& o, uint ind, RegExp *re)
+{
+	CharSet cs;
+	uint j;
+
 	re->split(cs);
 	/*
 	    for(uint k = 0; k < nChars;){
-		for(j = k; ++k < nChars && cs.rep[k] == cs.rep[j];);
+		for(j = k; ++k < nRealChars && cs.rep[k] == cs.rep[j];);
 		printSpan(cerr, j, k);
 		cerr << "\t" << cs.rep[j] - &cs.ptn[0] << endl;
 	    }
 	*/
-	Char rep[nChars];
+	Char *rep = new Char[nRealChars];
 
-	for (j = 0; j < nChars; ++j)
+	for (j = 0; j < nRealChars; ++j)
 	{
 		if (!cs.rep[j]->nxt)
 			cs.rep[j]->nxt = &cs.ptn[j];
@@ -895,10 +1051,11 @@ void genCode(std::ostream& o, RegExp *re)
 		}
 	}
 
-	DFA *dfa = new DFA(ins, re->size, 0, 256, rep);
-	dfa->emit(o);
+	DFA *dfa = new DFA(ins, re->size, 0, nRealChars, rep);
+	dfa->emit(o, ind);
 	delete dfa;
 	delete [] ins;
+	delete [] rep;
 }
 
 } // end namespace re2c
