@@ -1,9 +1,11 @@
-/* $Id: basics.h,v 1.5 2004/11/01 04:35:57 nuffer Exp $ */
+/* $Id: basics.h 674 2007-04-16 21:39:11Z helly $ */
 #ifndef _basics_h
 #define _basics_h
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
+#elif defined(_WIN32)
+#include "config_w32.h"
 #endif
 
 namespace re2c
