@@ -8,41 +8,60 @@
 namespace re2c
 {
 
-inline char octCh(uint c)
+void prtChOrHex(std::ostream& o, uint c, bool useTalx)
 {
-	return '0' + c % 8;
-}
+	int oc = (int)(re2c::wFlag || !useTalx ? c : re2c::talx[c]);
 
-inline char hexCh(uint c)
-{
-	const char * sHex = "0123456789ABCDEF";
-	
-	return sHex[c & 0x0F];
-}
-
-void prtChOrHex(std::ostream& o, uchar c)
-{
-	uchar oc = talx[c];
-
-	if (isprint(oc))
+	if ((oc < 256) && isprint(oc))
 	{
 		o << '\'';
-		prtCh(o, c);
+		prtCh(o, oc);
 		o << '\'';
 	}
 	else
 	{
-		o << "0x" << hexCh(c >> 4) << hexCh(c);
+		prtHex(o, c);
 	}
 }
 
-void prtCh(std::ostream &o, uchar c)
+void prtHex(std::ostream& o, uint c, bool useTalx)
 {
-	uchar oc = talx[c];
+	int oc = (int)(re2c::wFlag || !useTalx ? c : re2c::talx[c]);
+
+	if (re2c::uFlag)
+	{
+		o << "0x"
+		  << hexCh(oc >> 28)
+		  << hexCh(oc >> 24)
+		  << hexCh(oc >> 20)
+		  << hexCh(oc >> 16)
+		  << hexCh(oc >> 12)
+		  << hexCh(oc >>  8)
+		  << hexCh(oc >>  4)
+		  << hexCh(oc);
+	}
+	else if (re2c::wFlag)
+	{
+		o << "0x"
+		  << hexCh(oc >> 12)
+		  << hexCh(oc >>  8)
+		  << hexCh(oc >>  4)
+		  << hexCh(oc);
+	}
+	else
+	{
+		o << "0x"
+		  << hexCh(oc >>  4) 
+		  << hexCh(oc);
+	}
+}
+
+void prtCh(std::ostream& o, uint c, bool useTalx)
+{
+	int oc = (int)(re2c::wFlag || !useTalx ? c : re2c::talx[c]);
 
 	switch (oc)
 	{
-
 		case '\'':
 		o << "\\'";
 		break;
@@ -81,17 +100,41 @@ void prtCh(std::ostream &o, uchar c)
 
 		default:
 
-		if (isprint(oc))
+		if ((oc < 256) && isprint(oc))
+		{
 			o << (char) oc;
+		}
+		else if (re2c::uFlag)
+		{
+			o << "0x"
+			  << hexCh(oc >> 20)
+			  << hexCh(oc >> 16)
+			  << hexCh(oc >> 12)
+			  << hexCh(oc >>  8)
+			  << hexCh(oc >>  4)
+			  << hexCh(oc);
+		}
+		else if (re2c::wFlag)
+		{
+			o << "0x"
+			  << hexCh(oc >> 12)
+			  << hexCh(oc >>  8)
+			  << hexCh(oc >>  4)
+			  << hexCh(oc);
+		}
 		else
-			o << '\\' << octCh(c / 64) << octCh(c / 8) << octCh(c);
+		{
+			o << '\\' << octCh(oc / 64) << octCh(oc / 8) << octCh(oc);
+		}
 	}
 }
 
-void printSpan(std::ostream &o, uint lb, uint ub)
+void printSpan(std::ostream& o, uint lb, uint ub)
 {
 	if (lb > ub)
+	{
 		o << "*";
+	}
 
 	o << "[";
 
@@ -109,7 +152,7 @@ void printSpan(std::ostream &o, uint lb, uint ub)
 	o << "]";
 }
 
-uint Span::show(std::ostream &o, uint lb)
+uint Span::show(std::ostream &o, uint lb) const
 {
 	if (to)
 	{
@@ -125,16 +168,18 @@ std::ostream& operator<<(std::ostream &o, const State &s)
 	o << "state " << s.label;
 
 	if (s.rule)
+	{
 		o << " accepts " << s.rule->accept;
+	}
 
 	o << "\n";
 
-	++oline;
-
 	uint lb = 0;
 
 	for (uint i = 0; i < s.go.nSpans; ++i)
+	{
 		lb = s.go.span[i].show(o, lb);
+	}
 
 	return o;
 }
@@ -144,20 +189,29 @@ std::ostream& operator<<(std::ostream &o, const DFA &dfa)
 	for (State *s = dfa.head; s; s = s->next)
 	{
 		o << s << "\n\n";
-		++oline;
 	}
 
 	return o;
 }
 
-State::State() : rule(NULL), link(NULL), kCount(0), kernel(NULL), action(NULL)
+State::State()
+	: label(0)
+	, rule(NULL)
+	, next(0)
+	, link(NULL)
+	, depth(0)
+	, kCount(0)
+	, kernel(NULL)
+	, isPreCtxt(false)
+	, isBase(false)
+	, go()
+	, action(NULL)
 {
-	go.nSpans = 0;
-	go.span = NULL;
 }
 
 State::~State()
 {
+	delete action;
 	delete [] kernel;
 	delete [] go.span;
 }
@@ -174,7 +228,7 @@ static Ins **closure(Ins **cP, Ins *i)
 			cP = closure(cP, i + 1);
 			i = (Ins*) i->i.link;
 		}
-		else if (i->i.tag == GOTO)
+		else if (i->i.tag == GOTO || i->i.tag == CTXT)
 		{
 			i = (Ins*) i->i.link;
 		}
@@ -192,17 +246,18 @@ struct GoTo
 };
 
 DFA::DFA(Ins *ins, uint ni, uint lb, uint ub, Char *rep)
-		: lbChar(lb), ubChar(ub)
+	: lbChar(lb)
+	, ubChar(ub)
+	, nStates(0)
+	, head(NULL)
+	, tail(&head)
+	, toDo(NULL)
 {
 	Ins **work = new Ins * [ni + 1];
 	uint nc = ub - lb;
 	GoTo *goTo = new GoTo[nc];
 	Span *span = new Span[nc];
 	memset((char*) goTo, 0, nc*sizeof(GoTo));
-	tail = &head;
-	head = NULL;
-	nStates = 0;
-	toDo = NULL;
 	findState(work, closure(work, &ins[0]) - work);
 
 	while (toDo)
@@ -233,6 +288,10 @@ DFA::DFA(Ins *ins, uint ni, uint lb, uint ub, Char *rep)
 				if (!s->rule || ((RuleOp*) i->i.link)->accept < s->rule->accept)
 					s->rule = (RuleOp*) i->i.link;
 			}
+			else if (i->i.tag == CTXT)
+			{
+				s->isPreCtxt = true;
+			}
 		}
 
 		for (j = 0; j < nGoTos; ++j)
@@ -252,9 +311,8 @@ DFA::DFA(Ins *ins, uint ni, uint lb, uint ub, Char *rep)
 		{
 			State *to = (State*) goTo[rep[j]].to;
 
-			while (++j < nc && goTo[rep[j]].to == to)
+			while (++j < nc && goTo[rep[j]].to == to) ;
 
-				;
 			span[s->go.nSpans].ub = lb + j;
 
 			span[s->go.nSpans].to = to;
@@ -310,7 +368,7 @@ State *DFA::findState(Ins **kernel, uint kCount)
 
 	for (iP = kernel; (i = *iP); ++iP)
 	{
-		if (i->i.tag == CHAR || i->i.tag == TERM)
+		if (i->i.tag == CHAR || i->i.tag == TERM || i->i.tag == CTXT)
 		{
 			*cP++ = i;
 		}
