@@ -1,4 +1,4 @@
-/* $Id: ins.h,v 1.5 2004/11/01 04:35:57 nuffer Exp $ */
+/* $Id: ins.h 674 2007-04-16 21:39:11Z helly $ */
 #ifndef _ins_h
 #define _ins_h
 
@@ -7,8 +7,7 @@
 namespace re2c
 {
 
-const uint nChars = 256;
-typedef uchar Char;
+typedef unsigned short Char;
 
 const uint CHAR = 0;
 const uint GOTO = 1;
