@@ -2,7 +2,7 @@
  Author: Marcus Boerger <helly@users.sourceforge.net>
 */
 
-/* $Id: mbo_getopt.h,v 1.4 2005/03/13 03:08:35 helly Exp $ */
+/* $Id: mbo_getopt.h 674 2007-04-16 21:39:11Z helly $ */
 
 /* Define structure for one recognized option (both single char and long name).
  * If short_open is '-' this is the last option.
@@ -26,7 +26,7 @@ struct mbo_opt_struct
 	const char * opt_name;
 };
 
-int mbo_getopt(int argc, char* const *argv, const mbo_opt_struct opts[], char **optarg, int *optind, int show_err);
+int mbo_getopt(int argc, char* const *argv, const mbo_opt_struct *opts, char **optarg, int *optind, int show_err);
 
 } // end namespace re2c
 
