@@ -1,4 +1,4 @@
-/* $Id: parser.h,v 1.7 2005/10/25 22:02:43 helly Exp $ */
+/* $Id: parser.h 674 2007-04-16 21:39:11Z helly $ */
 #ifndef _parser_h
 #define _parser_h
 
@@ -19,30 +19,37 @@ public:
 	static Symbol *find(const SubStr&);
 	static void ClearTable();
 
-	~Symbol()
-	{
-		/** \todo should we delete 're'? */
-	}
+	typedef std::map<std::string, Symbol*> SymbolTable;
 
 protected:
 
 	Symbol(const SubStr& str)
-		: name(str)
-		, re(NULL)
+		: re(NULL)
+		, name(str)
 	{
 	}
 
 private:
 
-	typedef std::map<std::string, Symbol*> SymbolTable;
-
 	static SymbolTable symbol_table;
 
 	Str	name;
+
+#if PEDANTIC
+	Symbol(const Symbol& oth)
+		: re(oth.re)
+		, name(oth.name)
+	{
+	}
+	Symbol& operator = (const Symbol& oth)
+	{
+		new(this) Symbol(oth);
+		return *this;
+	}
+#endif
 };
 
-void line_source(unsigned int, std::ostream&);
-void parse(std::istream&, std::ostream&);
+void parse(Scanner&, std::ostream&);
 
 } // end namespace re2c
 
